#!/usr/bin/env python
# coding: utf-8

"""
    quickCrypt.py
    
    A small program to quickly and easily encrypt files using 
    the Phelix algorithm.  quickCrypt2 can process arbitrarily
    large files.  .......
    
    The key may be any string of up to 32 characters.  Be sure 
    to enclose the key in quotes if it has imbedded spaces.  
    Because the key is typed, some may consider the key weak.  
    Stronger keys are possible if you modify this program.  If
    the key is not on the commandline, the user will be asked 
    to type it in (no echo).
    
    The new encrypted file is named the same as the original 
    file but with a '.phx' suffix added.  The decrypted file 
    is written without the '.phx' suffix.  If the output file 
    already exists and is about to be overwritten, the user is
    asked for permission.
    
    Usage:
        python quick2Crypt.py <filename> [<key>]
    
    
    Larry Bugbee
    March 2007
    
"""

import pyPhelix, sys, md5, time, os, getpass, random, struct

#---------------------------------------------------------------
# constants

suffix = '.phx'

#---------------------------------------------------------------
# utilities

def okToProceed(filename):
    if os.path.exists(filename):
        prompt = '  "%s" exists.  overwrite? [y/n]: ' % filename
        response = raw_input(prompt)
        if not (response and response.lower()[0] == 'y'):
            print '  aborting: original file preserved'
            sys.exit()
    
# a quick way to get exactly 128 bits
def quick128(data):
    md = md5.md5()
    md.update(data)
    return md.digest()

def makeNounce():
    # sufficient time delay on most processors to ensure a 
    # unique nonce, but adding a supposedly random number can't
    # hurt
    time.sleep(0.1)
    nonce  = struct.pack('d', time.time())
    nonce += struct.pack('d', random.getrandbits(64))

    # probably unnecessary, but here are a few more identifiers 
    # that might further reduce the probability of time being 
    # reused.  
    #   ssid = os.environ.get('SECURITYSESSIONID', 'na')
    #   nonce += ' %d %d %d %s' % (os.getpid(), 
    #                              os.getsid(os.getpid()), 
    #                              os.getuid(),
    #                              ssid,
    #                             )
    return nonce

def usage():
    print '\n  usage:\n    python %s <filename> <key>\n' % sys.argv[0]
    sys.exit()

#---------------------------------------------------------------

def main():
    if len(sys.argv) > 1:
        filename = sys.argv[1]
    if len(sys.argv) == 3:
        key = sys.argv[2]
    elif len(sys.argv) == 2:
        key = getpass.getpass('  enter your key: ')
    else:
        usage()
    
    if len(key) > 32:
        print '\n  *** key must be 32 characters or less ***\n'
        sys.exit()
        
    encrypt = not filename.endswith(suffix)
    
    if encrypt:
        newfilename = filename+suffix
    else:
        newfilename = filename[:-len(suffix)]
    okToProceed(newfilename)
    fin  = open(filename)
    fout = open(newfilename, 'wb')
    
    if encrypt:
        nonce = makeNounce()
        IV    = quick128(nonce)
        fout.write(IV)
        ph = pyPhelix.Phelix(key, IV)
        while True:
            data = fin.read(4194304)  # 4MB (divisable by 4)
            if not data: break
            time.clock()
            ciphertext = ph.encryptBytes(data)
#            print time.clock()
            fout.write(ciphertext)
        mac = ph.getMAC()
        fout.write(mac)
    else:
        IV = fin.read(16)
        ph = pyPhelix.Phelix(key, IV)
        hold = ''
        while True:
            data = fin.read(4194304)  # 4MB (divisable by 4)
            if not data: break
            all = hold + data
            data, hold = all[:-16], all[-16:]
            if data:
                time.clock()
                plaintext = ph.decryptBytes(data)
#                print time.clock()
                fout.write(plaintext)
        mac = hold
        mac2 = ph.getMAC()
        if mac2 == mac:
            print '  *** good ***'
        else:
            print '  *** bad ***'
            ########################## need to delete file ???
    fin.close()
    fout.close()

#---------------------------------------------------------------

if __name__ == '__main__':
    main()
    
#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
